@@ -23,7 +23,18 @@ fitMeasures <- fitmeasures <- function(object, fit.measures="all") {
     N <- object@SampleStats@ntotal
     #q <- length(vnames(object@ParTable, "ov.x"))
     #p <- nvar - q
+
+    # Change 0.5-13: take into account explicit equality constraints!!
+    # reported by Mark L. Taper (affects AIC and BIC)
     npar <- object@Fit@npar
+    if(nrow(object@Model@con.jac) > 0L) {
+        ceq.idx <- attr(object@Model@con.jac, "ceq.idx")
+        if(length(ceq.idx) > 0L) {
+            neq <- qr(object@Model@con.jac[ceq.idx,,drop=FALSE])$rank
+            npar <- npar - neq
+        }
+    }
+
     fx <- object@Fit@fx
     fx.group <- object@Fit@fx.group
     meanstructure <- object@Model@meanstructure
@@ -266,7 +277,9 @@ fitMeasures <- fitmeasures <- function(object, fit.measures="all") {
                 t1 <- max( c(X2.scaled - df.scaled, 0) )
                 t2 <- max( c(X2.scaled - df.scaled,
                              X2.null.scaled - df.null.scaled, 0) )
-                if(t1 == 0 && t2 == 0) {
+                if(is.na(t1) || is.na(t2)){
+                    indices["cfi.scaled"] <- NA
+                } else if(t1 == 0 && t2 == 0) {
                     indices["cfi.scaled"] <- 1
                 } else {
                     indices["cfi.scaled"] <- 1 - t1/t2
@@ -328,9 +341,11 @@ fitMeasures <- fitmeasures <- function(object, fit.measures="all") {
                 if(df > 0) {
                     t1 <- X2.null.scaled/df.null.scaled - X2.scaled/df.scaled
                     t2 <- X2.null.scaled/df.null.scaled
-                    if(t1 < 0 || t2 < 0) {
+                    if(is.na(t1) || is.na(t2)) {
+                        RLI <- NA
+                    } else if(t1 < 0 || t2 < 0) {
                         RLI <- 1
-                    }     else {
+                    } else {
                         RLI <- t1/t2
                     }
                 } else {
@@ -381,7 +396,9 @@ fitMeasures <- fitmeasures <- function(object, fit.measures="all") {
             if("ifi.scaled" %in% fit.measures) {
                 t1 <- X2.null.scaled - X2.scaled
                 t2 <- X2.null.scaled
-                if(t2 < 0) {
+                if(is.na(t2)) {
+                    IFI <- NA
+                } else if(t2 < 0) {
                     IFI <- 1
                 } else {
                     IFI <- t1/t2
@@ -404,7 +421,9 @@ fitMeasures <- fitmeasures <- function(object, fit.measures="all") {
                 t1 <- X2.scaled - df.scaled
                 t2 <- X2.null.scaled - df.null.scaled
                 t2 <- X2.null - df.null
-                if(t1 < 0 || t2 < 0) {
+                if(is.na(t1) || is.na(t2)) {
+                    RNI <- NA
+                } else if(t1 < 0 || t2 < 0) {
                     RNI <- 1
                 } else {
                     RNI <- 1 - t1/t2
@@ -508,6 +527,7 @@ fitMeasures <- fitmeasures <- function(object, fit.measures="all") {
         } else if(df > 0) {
             if(scaled) {
                 d <- sum(object@Fit@test[[2]]$trace.UGamma)
+                if(d==0) d <- NA
             } 
             if(object@Options$mimic %in% c("Mplus", "lavaan")) {
                 GG <- 0
@@ -827,12 +847,12 @@ fitMeasures <- fitmeasures <- function(object, fit.measures="all") {
     }
 
     # do we have everything that we requested?
-    idx.missing <- which(is.na(match(fit.measures, names(indices))))
-    if(length(idx.missing) > 0L) {
-        cat("lavaan WARNING: some requested fit measure(s) are not available for this model:\n")
-        print( fit.measures[ idx.missing ] )
-        cat("\n")
-    }
+    #idx.missing <- which(is.na(match(fit.measures, names(indices))))
+    #if(length(idx.missing) > 0L) {
+    #    cat("lavaan WARNING: some requested fit measure(s) are not available for this model:\n")
+    #    print( fit.measures[ idx.missing ] )
+    #    cat("\n")
+    #}
     
     out <- unlist(indices[fit.measures])
 
